@@ -1,7 +1,7 @@
 /*
    Kickshaw - A Menu Editor for Openbox
 
-   Copyright (c) 2010–2022        Marcus Schätzle
+   Copyright (c) 2010–2024        Marcus Schätzle
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -115,9 +115,9 @@ static void add_startupnotify_or_execute_options_to_context_menu (      GtkWidge
                                                                   const guint8        number_of_opts, 
                                                                   const gchar       **options_array)
 {
-    const guint allocated_size = sizeof (gboolean) * number_of_opts;
-    // Used because VLAs cannot be initialized by any form of initialization syntax. 
-    g_autofree gboolean *opts_exist = g_malloc0 (allocated_size); // Initialize all elements to FALSE.
+    const gchar *menu_item_txts_exe_opts[] = { N_("Add Prompt"), N_("Add Command"), N_("Add Startupnotify") };
+    const gchar *menu_item_txts_snotify_opts[] = { N_("Add Enabled"), N_("Add Name"), N_("Add WM_CLASS") }; // Without Icon, because of context used below.s
+    g_autofree gboolean *opts_exist = g_malloc0 (sizeof (gboolean) * number_of_opts); // Initialize all elements to FALSE.
     GtkWidget *menu_item;
     gboolean add_new_callback;
 
@@ -125,17 +125,8 @@ static void add_startupnotify_or_execute_options_to_context_menu (      GtkWidge
 
     for (guint8 opts_cnt = 0; opts_cnt < number_of_opts; ++opts_cnt) {    
         if (!opts_exist[opts_cnt]) {
-            g_autofree gchar *menu_item_txt;
-
-            if (!STREQ (options_array[opts_cnt], "wmclass")) {
-                menu_item_txt = g_strconcat ("Add ", options_array[opts_cnt], NULL);
-                menu_item_txt[4] = g_ascii_toupper (menu_item_txt[4]);
-            }
-            else {
-                menu_item_txt = g_strdup ("Add WM_CLASS");
-            }
-
-            menu_item = gtk_menu_item_new_with_label (menu_item_txt);
+            menu_item = gtk_menu_item_new_with_label ((startupnotify_opts) ? ((opts_cnt < number_of_opts - 1) ? _(menu_item_txts_snotify_opts[opts_cnt]) : C_("Context Menu|Child of Startupnotify", "Add Icon")) : 
+                                                      _(menu_item_txts_exe_opts[opts_cnt]));
 
             add_new_callback = startupnotify_opts || opts_cnt != STARTUPNOTIFY;
             g_signal_connect_swapped (menu_item, "activate", 
@@ -148,7 +139,7 @@ static void add_startupnotify_or_execute_options_to_context_menu (      GtkWidge
 
 /* 
 
-    Right click on the treeview presents a context menu for adding, changing and deleting menu elements.
+    Right click on the treeview presents a context menu for adding, changing, and deleting menu elements.
 
 */
 
@@ -215,13 +206,14 @@ void create_context_menu (GdkEventButton *event)
                     }
                 }
                 if (!at_least_one_row_has_no_icon) {
-                    menu_item = gtk_menu_item_new_with_label ("Remove Icons");
+                    menu_item = gtk_menu_item_new_with_label (_("Remove Icons"));
                     g_signal_connect (menu_item, "activate", G_CALLBACK (remove_icons_from_menus_or_items), GUINT_TO_POINTER (TRUE));
                     gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
                     gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), gtk_separator_menu_item_new ());
                 }
             }
         }
+
         /*
             Either no or one row had been selected before or 
             two or more rows had been selected before and another row not one of these has now been rightclicked.
@@ -237,13 +229,14 @@ void create_context_menu (GdkEventButton *event)
             // Icon
             if (streq_any (ks.txt_fields[TYPE_TXT], "menu", "pipe menu", "item", NULL)) {
                 if (STREQ (ks.txt_fields[TYPE_TXT], "item")) {
-                    create_cm_headline (context_menu, " Icon"); // Items have "Icon" and "Action" as context menu headlines.
+                    create_cm_headline (context_menu, _(" Icon")); // Items have "Icon" and "Action" as context menu headlines.
                 }
-                menu_item = gtk_menu_item_new_with_label ((ks.txt_fields[ICON_PATH_TXT]) ? "Change Icon" : "Add Icon");
+
+                menu_item = gtk_menu_item_new_with_label ((ks.txt_fields[ICON_PATH_TXT]) ? _("Change Icon") : C_("Context Menu|Menu or Item", "Add Icon"));
                 g_signal_connect (menu_item, "activate", G_CALLBACK (icon_choosing_by_button_or_context_menu), NULL);
                 gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
                 if (ks.txt_fields[ICON_PATH_TXT]) {
-                    menu_item = gtk_menu_item_new_with_label ("Remove Icon");
+                    menu_item = gtk_menu_item_new_with_label (_("Remove Icon"));
                     g_signal_connect (menu_item, "activate", G_CALLBACK (remove_icons_from_menus_or_items), GUINT_TO_POINTER (TRUE));
                     gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
                 }
@@ -270,7 +263,9 @@ void create_context_menu (GdkEventButton *event)
                         const gint number_of_chldn_of_gp = gtk_tree_model_iter_n_children (ks.ts_model, &grandparent);
 
                         if (STREQ (ks.txt_fields[TYPE_TXT], "option block") && number_of_chldn_of_gp < NUMBER_OF_EXECUTE_OPTS) {
-                            g_autofree gchar *headline_txt = g_strdup_printf (" Startupnotify Option%s ", (gtk_tree_model_iter_n_children (ks.ts_model, &parent) == NUMBER_OF_STARTUPNOTIFY_OPTS - 1) ? "" : "s");
+                            gchar *headline_txt = ngettext (" Startupnotify Option", " Startupnotify Options", 
+                                                            (gtk_tree_model_iter_n_children (ks.ts_model, &parent) == NUMBER_OF_STARTUPNOTIFY_OPTS - 1));
+
                             create_cm_headline (context_menu, headline_txt);
                         }
 
@@ -304,7 +299,9 @@ void create_context_menu (GdkEventButton *event)
                         if (STREQ (ks.txt_fields[TYPE_TXT], "action") || 
                             (STREQ (ks.txt_fields[TYPE_TXT], "option block") && 
                             gtk_tree_model_iter_n_children (ks.ts_model, &ks.iter) < NUMBER_OF_STARTUPNOTIFY_OPTS)) {
-                            g_autofree gchar *headline_txt = g_strdup_printf (" Execute Option%s", (gtk_tree_model_iter_n_children (ks.ts_model, &parent) == NUMBER_OF_EXECUTE_OPTS - 1) ? "" : "s");
+                            gchar *headline_txt = ngettext (" Execute Option", " Execute Options", 
+                                                            (gtk_tree_model_iter_n_children (ks.ts_model, &parent) == NUMBER_OF_EXECUTE_OPTS - 1));
+
                             create_cm_headline (context_menu, headline_txt);
                         }
 
@@ -325,10 +322,10 @@ void create_context_menu (GdkEventButton *event)
                     streq_any (ks.txt_fields[MENU_ELEMENT_TXT], "Exit", "SessionLogout", "Restart", NULL) && 
                     !gtk_tree_model_iter_has_child (ks.ts_model, &ks.iter)) {
                     const gboolean restart = STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Restart");
-                    create_cm_headline (context_menu, (restart) ? " Restart Option" : 
+                    create_cm_headline (context_menu, (restart) ? _(" Restart Option") : 
                                         (STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Exit") ? 
-                                        " Exit Option" : " SessionLogout Option"));
-                    menu_item = gtk_menu_item_new_with_label ((restart) ? "Add Command" : "Add Prompt");
+                                        _(" Exit Option") : _(" SessionLogout Option")));
+                    menu_item = gtk_menu_item_new_with_label ((restart) ? _("Add Command") : _("Add Prompt"));
                     g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (add_new), (restart) ? "command" : "prompt");
                     gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
 
@@ -339,13 +336,13 @@ void create_context_menu (GdkEventButton *event)
             // Actions
             if (streq_any (ks.txt_fields[TYPE_TXT], "item", "action", NULL)) {
                 if (STREQ (ks.txt_fields[TYPE_TXT], "item") || not_all_options_of_selected_action_set) {
-                    create_cm_headline (context_menu, " Actions");
+                    create_cm_headline (context_menu, _(" Actions"));
                 }
 
                 for (guint8 actions_cnt = 0; actions_cnt < NUMBER_OF_ACTIONS; ++actions_cnt) {
-                    g_autofree gchar *menu_item_txt = g_strconcat ("Add ", ks.actions[actions_cnt], NULL);
+                    const gchar *menu_item_txts[] = { N_("Add Execute"), N_("Add Exit"), N_("Add Reconfigure"), N_("Add Restart"), N_("Add SessionLogut") };
 
-                    menu_item = gtk_menu_item_new_with_label (menu_item_txt);
+                    menu_item = gtk_menu_item_new_with_label (_(menu_item_txts[actions_cnt]));
 
                     g_signal_connect_swapped (menu_item, "activate", 
                                               G_CALLBACK ((actions_cnt == RECONFIGURE) ? 
@@ -367,7 +364,9 @@ void create_context_menu (GdkEventButton *event)
         If number of rows is > 1, then txt_fields[ELEMENT_VISIBILITY_TXT] == NULL.
     */
     if (!path || ks.txt_fields[ELEMENT_VISIBILITY_TXT]) {
-        const char *basic_menu_elements[] = { "menu", "pipe menu", "item", "separator" };
+        const gchar *basic_menu_elements[] = { "menu", "pipe menu", "item", "separator" };
+        const gchar *add_basic_menu_elements_for_cm[] = { N_("Add Menu"), N_("Add Pipe Menu"), N_("Add Item"), N_("Add Separator") };
+        const gchar *basic_menu_elements_for_cm[] = { N_("Menu"), N_("Pipe Menu"), N_("Item"), N_("Separator") };
         const gchar *items[] = { "item+Execute", "item+Exit", "item+Reconfigure", "item+Restart", "item+SessionLogout" };
         enum { MENU, PIPE_MENU, ITEM, SEPARATOR };
         const guint8 number_of_basic_menu_elements = G_N_ELEMENTS (basic_menu_elements);
@@ -375,24 +374,14 @@ void create_context_menu (GdkEventButton *event)
         GtkWidget *item_submenu = gtk_menu_new ();
         GtkWidget *submenu_item;
 
-        guint8 toupper_pos;
-
         if (!path) {
             gtk_tree_selection_unselect_all (selection);
-            create_cm_headline (context_menu, " Add at Top Level");
+            create_cm_headline (context_menu, _(" Add at Top Level"));
         }
 
         for (guint8 basic_menu_elements_cnt = 0; basic_menu_elements_cnt < number_of_basic_menu_elements; ++basic_menu_elements_cnt) {
-            g_autofree gchar *menu_item_txt = g_strconcat ((path) ? "Add " : "", basic_menu_elements[basic_menu_elements_cnt], NULL);
-
-            toupper_pos = (path) ? 4 : 0;
-
-            menu_item_txt[toupper_pos] = g_ascii_toupper (menu_item_txt[toupper_pos]);
-
-            if (basic_menu_elements_cnt == PIPE_MENU) {
-                // pipe menu -> Pipe Menu
-                menu_item_txt[toupper_pos + 5] =  g_ascii_toupper (menu_item_txt[toupper_pos + 5]);
-            }
+            const gchar *menu_item_txt = (path) ? _(add_basic_menu_elements_for_cm[basic_menu_elements_cnt]) : 
+                                                  _(basic_menu_elements_for_cm[basic_menu_elements_cnt]);
 
             menu_item = gtk_menu_item_new_with_label (menu_item_txt);
 
@@ -413,7 +402,7 @@ void create_context_menu (GdkEventButton *event)
                     g_signal_connect_swapped (submenu_item, "activate", G_CALLBACK (add_new), (gpointer) items[submenu_elements_cnt]);
                 }
 
-                submenu_item = gtk_menu_item_new_with_label ("No Action");
+                submenu_item = gtk_menu_item_new_with_label (_("No Action"));
                 gtk_menu_shell_append (GTK_MENU_SHELL (item_submenu), submenu_item);
                 g_signal_connect_swapped (submenu_item, "activate", G_CALLBACK (add_new), "item w/o action");
 
@@ -424,7 +413,7 @@ void create_context_menu (GdkEventButton *event)
             gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), gtk_separator_menu_item_new ());
         }
         
-		gtk_menu_set_reserve_toggle_size (GTK_MENU (item_submenu), FALSE);        
+        gtk_menu_set_reserve_toggle_size (GTK_MENU (item_submenu), FALSE);        
     }
 
     if (path) {
@@ -437,7 +426,7 @@ void create_context_menu (GdkEventButton *event)
         GtkTreePath *path_loop;
 
         // Remove
-        menu_item = gtk_menu_item_new_with_label ("Remove");
+        menu_item = gtk_menu_item_new_with_label (_("Remove"));
         gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
         g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (remove_rows), "context menu");
 
@@ -463,7 +452,7 @@ void create_context_menu (GdkEventButton *event)
 
         if (!node_statuses.at_least_one_selected_node_has_no_children) {
             // Remove all children
-            menu_item = gtk_menu_item_new_with_label ("Remove All Children");
+            menu_item = gtk_menu_item_new_with_label (_("Remove All Children"));
             g_signal_connect (menu_item, "activate", G_CALLBACK (remove_all_children), NULL);
             gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
 
@@ -477,10 +466,12 @@ void create_context_menu (GdkEventButton *event)
                      expansion_statuses_of_subnodes.at_least_one_imd_ch_is_expanded)) || 
                     (exp_status_changes_cnt == CM_COLLAPSE && node_statuses.at_least_one_selected_node_is_expanded)) {
                     g_autofree gchar *menu_item_txt = g_strdup_printf ((exp_status_changes_cnt == CM_RECURSIVELY) ? 
-                                                                       "Expand Row%s Recursively" : 
+                                                                       ngettext ("Expand Row Recursively", "Expand Rows Recursively", 
+                                                                                 (number_of_selected_rows == 1)) : 
                                                                        ((exp_status_changes_cnt == CM_IMMEDIATE) ? 
-                                                                        "Expand Row%s (Imd. Ch. Only)" : "Collapse Row%s"), 
-                                                                       (number_of_selected_rows == 1) ? "" : "s");
+                                                                        ngettext ("Expand Row (Imd. Ch. Only)", "Expand Rows (Imd. Ch. Only)", 
+                                                                                  (number_of_selected_rows == 1)) :
+                                                                        ngettext ("Collapse Row", "Collapse Rows", (number_of_selected_rows == 1))));
                     menu_item = gtk_menu_item_new_with_label (menu_item_txt);
 
                     g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (expand_or_collapse_selected_rows), 
@@ -490,7 +481,7 @@ void create_context_menu (GdkEventButton *event)
             }
         }
 
-        // Visualization of invisible menus, pipe menus, items and separators.
+        // Visualization of invisible menus, pipe menus, items, and separators.
         FOREACH_IN_LIST (selected_rows, selected_rows_loop) {
             g_autofree gchar *element_visibility_txt_loop;
 
@@ -518,8 +509,7 @@ void create_context_menu (GdkEventButton *event)
 
             for (guint8 visualization_cnt = VISUALIZE; visualization_cnt <= VISUALIZE_CM_RECURSIVELY; ++visualization_cnt) {
                 if (!(visualization_cnt == VISUALIZE_CM_RECURSIVELY && !at_least_one_descendant_is_invisible)) {
-                    g_autofree gchar *menu_item_txt = g_strdup_printf ("Visualize%s", (visualization_cnt == VISUALIZE) ? "" : " Recursively");
-                    menu_item = gtk_menu_item_new_with_label (menu_item_txt);
+                    menu_item = gtk_menu_item_new_with_label ((visualization_cnt == VISUALIZE) ? _("Visualize") : _("Visualize Recursively"));
                     g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (visualize_menus_items_and_separators), 
                                               GUINT_TO_POINTER ((guint) visualization_cnt)); // recursively = 1 = TRUE.
                     gtk_menu_shell_append (GTK_MENU_SHELL (context_menu), menu_item);
@@ -528,7 +518,7 @@ void create_context_menu (GdkEventButton *event)
         }
     }
 
-	gtk_menu_set_reserve_toggle_size (GTK_MENU (context_menu), FALSE); // Hide space reserved for toggle symbols as this is not needed inside the context menu.
+    gtk_menu_set_reserve_toggle_size (GTK_MENU (context_menu), FALSE); // Hide space reserved for toggle symbols as this is not needed inside the context menu.
 
     gtk_widget_show_all (context_menu);
 
